// QRSS Transmitter from AD9851
// George Smart, M1GEO.
//  here: http://www,george-smart.co.uk/wiki/Arduino_QRSS
//
// AD9851 Code
// Originally by Peter Marks
//  here: http://blog.marxy.org/2008/05/controlling-ad9851-dds-with-arduino.html
//

// DDS Reference Oscilliator Frequency, in Hz. (Remember, the PLL).
#define DDS_REF   180000000

// DDS Offset in Hz
#define DDS_OSET  404

// QRSS Output Frequency
#define QRSS_TX  10.140040e6

// QRSS Offset Frequency
#define QRSS_OSET 5

// QRSS Timings in milliseconds
#define QRSS_DIT  6000   //5000
#define QRSS_DAH  (QRSS_DIT*3)  //18000

// Triange Ramp time in milliseconds
#define TRI_TIME  (2*QRSS_DIT)

// DDS/Arduino Connections
#define DDS_LOAD  8
#define DDS_CLOCK 9
#define DDS_DATA  10
#define LED       13
#define QRSS_TXF  (QRSS_TX+DDS_OSET)
#include <stdint.h>

void setup()
{
  // Set all pins to output states
  pinMode (DDS_DATA,  OUTPUT);
  pinMode (DDS_CLOCK, OUTPUT);
  pinMode (DDS_LOAD,  OUTPUT);
  pinMode (LED,       OUTPUT);
  
  // Setup RS232
  Serial.begin(9600);
  
  // let the radio settle a bit before we start sending data
  delay(100);
  frequency(QRSS_TXF);
  waitDit();
}

void loop()
{
  Serial.println("*** Starting Cycle");
  
  waitDah();
  
  Serial.print("Sending Morse: ");
  sendPattern("--/.----/--././---"); //M1GEO in morse.
  Serial.println(" OK");

  Serial.print("Sending 3 Triangles: ");
  // Send 3 up ramps
  for(int i=0;i<3;i++) {
    Serial.print(i+1);
    digitalWrite(LED, HIGH);
    sendUpRamp();
    digitalWrite(LED, LOW);
    sendDownRamp();
    Serial.print(" ");
  }
  Serial.println("OK");

  waitDah();
  waitDah();
  waitDah();
}

void frequency(unsigned long frequency) {
  unsigned long tuning_word = (frequency * pow(2, 32)) / DDS_REF;
  digitalWrite (DDS_LOAD, LOW); // take load pin low

  for(int i = 0; i < 32; i++) {
    if ((tuning_word & 1) == 1)
      outOne();
    else
      outZero();
    tuning_word = tuning_word >> 1;
  }
  byte_out(0x09);
  digitalWrite (DDS_LOAD, HIGH); // Take load pin high again
}

void byte_out(unsigned char byte) {
  int i;

  for (i = 0; i < 8; i++) {
    if ((byte & 1) == 1)
      outOne();
    else
      outZero();
    byte = byte >> 1;
  }
}

void outOne() {
  digitalWrite(DDS_CLOCK, LOW);
  digitalWrite(DDS_DATA, HIGH);
  digitalWrite(DDS_CLOCK, HIGH);
  digitalWrite(DDS_DATA, LOW);
}

void outZero() {
  digitalWrite(DDS_CLOCK, LOW);
  digitalWrite(DDS_DATA, LOW);
  digitalWrite(DDS_CLOCK, HIGH);
}

void sendDit() {
  frequency(QRSS_TXF + QRSS_OSET);
  waitDit();
  frequency(QRSS_TXF);
  waitDit();  // always end on a dit duration
}

void sendDah() {
  frequency(QRSS_TXF + QRSS_OSET);
  waitDah();
  frequency(QRSS_TXF);
  waitDit();  // always end on a dit duration
}

void waitDit() {
  int a = 0;
  for (a=0;a<QRSS_DIT;a+=100) {
    digitalWrite(LED, HIGH);
    delay(25);
    digitalWrite(LED, LOW);
    delay(75);
  }
}

void waitDah() {
  int a = 0;
  for (a=0;a<QRSS_DAH;a+=100) {
    digitalWrite(LED, HIGH);
    delay(75);
    digitalWrite(LED, LOW);
    delay(25);
  }
}

void sendPattern(String str) {
  int strLen = str.length();
  Serial.print(strLen);
  Serial.print(" elements: ");
  
  int i=0;
  for(i=0;i<strLen;i++) {
    switch (str.charAt(i)) {
      case '-':
        Serial.print("-");
        sendDah();
        break;
      case '.':
        Serial.print(".");
        sendDit();
        break;
      case '/':
      case ' ':
        Serial.print("/");
        waitDah();
        break;
      default: 
        Serial.print("(I didn't recognise '");
        Serial.print(str.charAt(i));
        Serial.print("'.  Permitted Chars are -  .  or  /)");
    }
  }
}

void sendDownRamp() {
  for (int i=QRSS_OSET; i>=0;i--) {
    frequency(QRSS_TXF + i);
    delay(TRI_TIME/QRSS_OSET);
  }
}

void sendUpRamp() {
  for (int i=0; i<=QRSS_OSET;i++) {
    frequency(QRSS_TXF + i);
    delay(TRI_TIME/QRSS_OSET);
  }
}

